Unit PakKeUn;
(*
   Bitvektoros halmazbrzols pldja.
   Magyar krtya-halmaz tpusa.
   Asszocilt mveletek:      (Kszsg)
   -------------- THalmaz(TKartya) ---------------
   * TeljesPakli                  +
   * UresPakli                    -
   * TeljesPakliE                 -
   * UresPakliE                   +
   * Kozetesz                     +
   * Elvesz                       +
   * BenneVanE                    -
   * Egyesit                      -
   * Kozosek                      -
   * LegjobbLap                   +
   * LegrosszabbLap               -
   * VeletlenLap                  +
   . . . . . .
   * BePakli                      -
   * KiPakli                      +
   * HibasEPakli                  +
*)
  Interface
    Uses
      KartyaUn,SzinUnit,FiguUnit;
    Type
      THalmaz = Record
                  lapok: Array [TSzin,TFigura] of Boolean;
                  siker: Boolean;
                End;
    Const
      MaxKartyaDb = (Ord(MaxFigura)+1)*(Ord(MaxSzin)+1);

    Procedure TeljesPakli(Var p:THalmaz);
    {Ef: -
     Uf: MINDEN s:TSzin, MINDEN f:TFigura: p.lapok[s,f] ES
         p.siker}
    Function TeljesPakliE(Const p:THalmaz):Boolean;
    {Ef: -
     Uf: TeljesPakliE(p) <=> MINDEN s:TSzin, MINDEN f:TFigura:
                                                          p.lapok[s,f]}
    Procedure UresPakli(Var p:THalmaz);
    {Ef: -
     Uf: MINDEN s:TSzin, MINDEN f:TFigura: NEM p.lapok[s,f] ES
         p.siker}
    Function UresPakliE(Var p:THalmaz):Boolean;
    {Ef: -
     Uf: UresPakliE(p) <=> MINDEN s:TSzin, MINDEN f:TFigura:
                                                       NEM p.lapok[s,f]}
    Procedure Kozetesz(Var p:THalmaz; Const k:TKartya);
    {Ef: -
     Uf: k=(s,f) ES p.lapok[s,f]}
    Procedure Elvesz(Var p:THalmaz; Const k:TKartya);
    {Ef: k=(s,f) ES p.lapok[s,f]
     Uf: k=(s,f) ES NEM p.lapok[s,f]}
    Function  BenneVanE(Var p:THalmaz; Const k:TKartya):Boolean;
    {Ef: -
     Uf: BenneVanE(p,k) <=> k=(s,f) ES p.lapok[s,f] }
    Procedure Egyesit(Const p1,p2:THalmaz; Var p:THalmaz);
    {Ef: -
     Uf: ...}
    Procedure Kozosek(Const p1,p2:THalmaz; Var p:THalmaz);
    {Ef: -
     Uf: ...}
    Procedure LegjobbLap(Var p:THalmaz; Var k:TKartya);
    {Ef: NEM UresPakliE(p)
     Uf: k=(s,f) ES MINDEN kk:TKartya: kk=(ss,ff) ES
                                       BenneVanE(ss,ff) ES
                                       JobbE((ss,ff),(s,f))}
    Procedure LegrosszabbLap(Var p:THalmaz; Var k:TKartya);
    {Ef: NEM UresPakliE(p)
     Uf: ...}
    Procedure VeletlenLap(Var p:THalmaz; Var k:TKartya);
    {Ef: NEM UresPakliE(p)
     Uf: BenneVan(p,k)}
    Procedure BePakli(Const kerd:String; Var p:THalmaz);
    {Ef: -
     Uf: ...}
    Procedure KiPakli(Const szov:String; Const p:THalmaz);
    {Ef: -
     Uf: ...}
    Function HibasEPakli(Var p:THalmaz):Boolean;
    {Ef: -
     Uf: HibasEPakli=p.siker ES p'.siker}

  Implementation

    Uses {Newdelay,}Crt; {az AltRutin.inc kvnja meg}
    {$i AltRutin.inc}

    {------------------------- KrtyaHalmaz: --------------------------}

    Procedure TeljesPakli(Var p:THalmaz);
    {Ef: -
     Uf: MINDEN s:TSzin, MINDEN f:TFigura: p.lapok[s,f] ES
         p.siker}
      Var
        s:TSzin; f:TFigura;
    Begin
      For s:=MinSzin to MaxSzin do
      Begin
        For f:=MinFigura to MaxFigura do
        Begin
          p.lapok[s,f]:=True
        End;
      End;
      p.siker:=True
    End;{TeljesPakli}

    Function TeljesPakliE(Const p:THalmaz):Boolean;
    {Ef: -
     Uf: TeljesPakliE(p) <=> MINDEN s:TSzin, MINDEN f:TFigura:
                                                          p.lapok[s,f]}
      Var
        s:TSzin; f:TFigura; db:Byte;
    Begin
      {... hf ...}
    End;{TeljesPakliE}

    Procedure UresPakli(Var p:THalmaz);
    {Ef: -
     Uf: MINDEN s:TSzin, MINDEN f:TFigura: NEM p.lapok[s,f] ES
         p.siker}
      Var
        s:TSzin; f:TFigura;
    Begin
      {... hf ...}
    End;{UresPakli}

    Function UresPakliE(Var p:THalmaz):Boolean;
    {Ef: -
     Uf: UresPakliE(p) <=> MINDEN s:TSzin, MINDEN f:TFigura:
                                                     NEM p.lapok[s,f]}
      Var
        s:TSzin; f:TFigura; db:Byte;
    Begin
      db:=0;
      For s:=MinSzin to MaxSzin do
      Begin
        For f:=MinFigura to MaxFigura do
        Begin
          If p.lapok[s,f] then Inc(db)
        End;
      End;
      UresPakliE:=db=0
    End;{UresPakliE}

    Procedure Kozetesz(Var p:THalmaz; Const k:TKartya);
    {Ef: -
     Uf: k=(s,f) ES p.lapok[s,f]}
    Begin
      p.lapok[k.szin,k.figura]:=True
    End;{Kozetesz}

    Procedure Elvesz(Var p:THalmaz; Const k:TKartya);
    {Ef: k=(s,f) ES p.lapok[s,f]
     Uf: k=(s,f) ES NEM p.lapok[s,f]}
    Begin
      If not p.lapok[k.szin,k.figura] then {nem teljesl az Ef}
      Begin
        p.siker:=False
      End
        Else
      Begin
        p.lapok[k.szin,k.figura]:=False
      End;
    End;{Elvesz}

    Function  BenneVanE(Var p:THalmaz; Const k:TKartya):Boolean;
    {Ef: -
     Uf: BenneVanE(p,k) <=> k=(s,f) ES p.lapok[s,f]}
    Begin
      {... hf ...}
    End;{BenneVanE}

    Procedure Egyesit(Const p1,p2:THalmaz; Var p:THalmaz);
    {Ef: -
     Uf: ...}
    Begin
      {... hf ...}
    End;{Egyesit}

    Procedure Kozosek(Const p1,p2:THalmaz; Var p:THalmaz);
    {Ef: -
     Uf: ...}
    Begin
      {... hf ...}
    End;{Kozosek}

    Procedure LegjobbLap(Var p:THalmaz; Var k:TKartya);
    {Ef: NEM UresPakliE(p)
     Uf: ...}
      Var
        s:TSzin; f:TFigura;
    Begin
      If UresPakliE(p) then {nem teljesl az Ef}
      Begin
        p.siker:=False
      End
        Else
      Begin
        s:=Zold; f:=Asz;
        While ((s>Makk) or (s=Makk) and (f>VII)) and
              not p.lapok[s,f] do
        Begin
          If f>VII then f:=pred(f)
                   else Begin s:=pred(s); f:=Asz end;
        End;
        k.szin:=s; k.figura:=f;
      End;
    End;{LegjobbLap}

    Procedure LegrosszabbLap(Var p:THalmaz; Var k:TKartya);
    {Ef: NEM UresPakliE(p)
     Uf: k=(s,f) ES MINDEN kk:TKartya: kk=(ss,ff) ES
                                       BenneVanE(ss,ff) ES
                                       RosszabbE((ss,ff),(s,f))}
      Var
        s:TSzin; f:TFigura;
    Begin
      If UresPakliE(p) then {nem teljesl az Ef}
      Begin
        p.siker:=False
      End
        Else
      Begin
        {... hf ...}
      End;
    End;{LegrosszabbLap}

    Procedure VeletlenLap(Var p:THalmaz; Var k:TKartya);
    {Ef: NEM UresPakliE(p)
     Uf: BenneVan(p,k)}
      Var
        s:TSzin; f:TFigura;
    Begin
      If UresPakliE(p) then {nem teljesl az Ef}
      Begin
        p.siker:=False
      End
        Else
      Begin
        Repeat
          s:=Kod2Szin(Random(4)); f:=Kod2Figura(Random(8));
        Until p.lapok[s,f];
        k.szin:=s; k.figura:=f
      End;
    End;{VeletlenLap}

    Procedure BePakli(Const kerd:String; Var p:THalmaz);
    {Ef: ...
     Uf: ...}
    Begin
      {... hf ...}
    End;{BePakli}

    Procedure KiPakli(Const szov:String; Const p:THalmaz);
    {Ef: -
     Uf: ...}
      Var
        s:TSzin; f:TFigura;
    Begin
      Writeln(szov);
      For s:=MinSzin to MaxSzin do
      Begin
        Write(Szin2String(s):6,':');
        For f:=MinFigura to MaxFigura do
        Begin
          If p.lapok[s,f] then Write(Figura2String(f),' ')
        End;
        Writeln;
      End;
    End;{KiPakli}

    Function HibasEPakli(Var p:THalmaz):Boolean;
    {Ef: -
     Uf: HibasEPakli=p.siker ES p'.siker }
    Begin
      HibasEPakli:=p.siker;
      p.siker:=True
    End;{HibasEPakli}

Begin

End.
